package com.persagy.proxy.adm.controller;

import com.persagy.proxy.adm.constant.AdmCommonConstant;
import com.persagy.proxy.adm.model.AdmManualRelCalRequest;
import com.persagy.proxy.adm.model.AdmManualRelationCalcDel;
import com.persagy.proxy.adm.model.AdmRelationAddRequest;
import com.persagy.proxy.adm.request.AdmCreateResponse;
import com.persagy.proxy.adm.request.AdmManualRelCalResponse;
import com.persagy.proxy.adm.request.AdmResponse;
import com.persagy.proxy.adm.service.AdmManualRelCalService;
import com.persagy.proxy.common.entity.InstanceUrlParam;
import lombok.RequiredArgsConstructor;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.ws.rs.QueryParam;

/***
 * Description:手动计算输入类
 * @author : lijie
 * @date :2021/8/31 11:37
 * Update By lijie 2021/8/31 11:37
 */
@RestController
@RequestMapping("/rel-manual-calc/")
@RequiredArgsConstructor
public class AdmManualRelCalController {

    private final AdmManualRelCalService admManualRelationCalcService;
    @Value("${middleware.group.code}")
    private String groupCode;
    
    /***
     * Description: 添加关系
     * @param request : 请求参数
     * @return : com.persagy.proxy.adm.request.AdmCreateResponse  
     * @author : lijie
     * @date :2021/8/31 12:10
     * Update By lijie 2021/8/31 12:10
     */
    @PostMapping(value = "add")
    public AdmCreateResponse addRels(@QueryParam("projectId") String projectId,
                                     @RequestBody AdmRelationAddRequest request) throws Exception {
        // 组装上下文条件
        InstanceUrlParam context = new InstanceUrlParam(groupCode, null, projectId, AdmCommonConstant.APP_ID);
        return admManualRelationCalcService.addRels(context,request);
    }
    /***
     * Description: 删除关系
     * @param admManualRelationCalcDel : 请求参数
     * @return : com.persagy.proxy.adm.request.AdmCreateResponse
     * @author : lijie
     * @date :2021/8/31 12:10
     * Update By lijie 2021/8/31 12:10
     */
    @PostMapping(value = "del")
    public AdmCreateResponse delRels(@QueryParam("projectId") String projectId,
                                     @Validated @RequestBody AdmManualRelationCalcDel admManualRelationCalcDel) throws Exception {
        // 组装上下文条件
        InstanceUrlParam context = new InstanceUrlParam(groupCode, null, projectId, AdmCommonConstant.APP_ID);
        return admManualRelationCalcService.delRels(context,admManualRelationCalcDel);
    }
    /***
     * Description: 查询关系数据
     * @param admManualRelCalRequest : 请求参数
     * @return : com.persagy.proxy.adm.request.AdmCreateResponse
     * @author : lijie
     * @date :2021/8/31 12:10
     * Update By lijie 2021/8/31 12:10
     */
    @PostMapping(value = "query")
    public AdmResponse query(@QueryParam("projectId") String projectId,
                             @Validated @RequestBody AdmManualRelCalRequest admManualRelCalRequest) throws Exception {
        // 组装上下文条件
        InstanceUrlParam context = new InstanceUrlParam(groupCode, null, projectId, AdmCommonConstant.APP_ID);
        return admManualRelationCalcService.query(context,admManualRelCalRequest);
    }




}
