package com.persagy.proxy.adm.controller;

import com.persagy.proxy.adm.constant.AdmCommonConstant;
import com.persagy.proxy.adm.model.AdmEquipment;
import com.persagy.proxy.adm.model.AdmMepSourceTypeEntity;
import com.persagy.proxy.adm.request.AdmCreateRequest;
import com.persagy.proxy.adm.request.AdmQueryCriteria;
import com.persagy.proxy.adm.request.AdmResponse;
import com.persagy.proxy.adm.service.AdmMepSystemCalcService;
import com.persagy.proxy.common.entity.InstanceUrlParam;
import lombok.RequiredArgsConstructor;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.web.bind.annotation.*;

import javax.ws.rs.QueryParam;

/***
 * Description: 机电管网系统关系计算
 * @author : lijie
 * @date :2021/9/14 23:15
 * Update By lijie 2021/9/14 23:15
 */
@RestController
@RequestMapping("/mep-system-calc/source/")
@RequiredArgsConstructor
public class AdmMepSystemCalcController {

    private final AdmMepSystemCalcService admMepSystemCalcService;

    @Value("${middleware.group.code}")
    private String groupCode;

    /***
     * Description: 添加源端设备类型
     * @param projectId : 项目id
     * @param request : 请求参数
     * @return : com.persagy.proxy.adm.request.AdmResponse
     * @author : lijie
     * @date :2021/9/14 23:20
     * Update By lijie 2021/9/14 23:20
     */
    @PostMapping("add")
    public AdmResponse add(@QueryParam("projectId") String projectId,
                           @RequestBody AdmCreateRequest<AdmMepSourceTypeEntity> request) throws Exception {
        // 组装上下文条件
        InstanceUrlParam context = new InstanceUrlParam(groupCode, null, projectId, AdmCommonConstant.APP_ID);
        return admMepSystemCalcService.add(context, request);
    }
    /***
     * Description: 查询源端设备类型
     * @param projectId : 项目id
     * @param calcName : 关系名称
     * @return : com.persagy.proxy.adm.request.AdmResponse
     * @author : lijie
     * @date :2021/9/14 23:20
     * Update By lijie 2021/9/14 23:20
     */
    @GetMapping("query")
    public AdmResponse query(@QueryParam("projectId") String projectId,
                             @QueryParam("calcName") String calcName) throws Exception {
        // 组装上下文条件
        InstanceUrlParam context = new InstanceUrlParam(groupCode, null, projectId, AdmCommonConstant.APP_ID);
        return admMepSystemCalcService.query(context, calcName);
    }


}
