package com.persagy.proxy.adm.controller;

import cn.hutool.core.util.StrUtil;
import com.persagy.dmp.common.context.AppContext;
import com.persagy.proxy.adm.model.AdmProject;
import com.persagy.proxy.adm.request.AdmQueryCriteria;
import com.persagy.proxy.adm.request.AdmResponse;
import com.persagy.proxy.adm.service.IAdmProjectService;
import com.persagy.proxy.common.entity.InstanceUrlParam;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.ws.rs.QueryParam;

/**
 * ADM 项目 API
 * @author Charlie Yu
 * @date 2021-08-16
 */
@RestController
@RequestMapping("/object/project")
public class AdmProjectController {

    @Autowired
    private IAdmProjectService service;

    @Value("${middleware.group.code}")
    private String groupCode;

    /**
     * 查询项目信息
     * @param request
     * @return
     * @throws Exception
     */
    @PostMapping("/query")
    public AdmResponse query(@QueryParam("projectId") String projectId, @RequestBody AdmQueryCriteria request) throws Exception {
        request.setName(AdmProject.OBJ_TYPE);
        String group = (StrUtil.isBlank(AppContext.getContext().getGroupCode()) || AppContext.getContext().getGroupCode().equals("null") ) ? groupCode:AppContext.getContext().getGroupCode();
        InstanceUrlParam param = InstanceUrlParam.builder().groupCode(group)
                .projectId(projectId).userId(AppContext.getContext().getAccountId())
                .appId(AppContext.getContext().getAppId()).build();
        return service.doQuery(param, request, AdmProject.class);
    }

    /**
     * 查询项目信息-全部或指定id
     * @param request
     * @return
     * @throws Exception
     */
    @PostMapping("/query/obj")
    public AdmResponse queryObj(@RequestBody AdmQueryCriteria request) throws Exception {
        return query(null, request);
    }
}
