package com.persagy.proxy.adm.controller;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.util.StrUtil;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.fasterxml.jackson.databind.node.JsonNodeFactory;
import com.fasterxml.jackson.databind.node.ObjectNode;
import com.persagy.dmp.basic.model.QueryCriteria;
import com.persagy.proxy.adm.constant.AdmCommonConstant;
import com.persagy.proxy.adm.request.AdmResponse;
import com.persagy.proxy.adm.service.IAdmRelationService;
import com.persagy.proxy.common.entity.InstanceUrlParam;
import com.persagy.proxy.common.entity.RelationDTO;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.web.bind.annotation.*;

import java.util.ArrayList;
import java.util.List;

/**
 * @description: 021关系-楼层的贯通
 * @author:lgy
 * @data:2021/9/13 11:02
 */
@RestController
@RequestMapping("/rel/fl-through-fl")
public class RelationFlThroughFlController {
    @Autowired
    private IAdmRelationService service;
    @Value("${middleware.group.code}")
    private String groupCode;
    /**
     * 创建楼层的贯通关系
     * @param param
     * @return
     * @throws Exception
     */
    @PostMapping(value = {"/link"})
    public AdmResponse create(@RequestParam("projectId") String projectId,
                              @RequestBody JSONObject param) throws Exception {
        String floorId = param.getString("floorId");
        String floorOtherId = param.getString("floorOtherId");
        if(!StrUtil.isAllNotEmpty(floorId,floorOtherId)) {
            return AdmResponse.failure("必填项：FloorId（楼层id）、FloorOtherId（其他楼层id）");
        }
        // 创建关系对象
        List<RelationDTO> voList = new ArrayList<>();
        voList.add(new RelationDTO(null, "ThroughRelationship", "Fl2Fl", null, floorId , floorOtherId));
        // 组装上下文条件
        InstanceUrlParam context = new InstanceUrlParam(groupCode, null, projectId, AdmCommonConstant.APP_ID);
        service.doSave(context, voList);
        return AdmResponse.success();
    }
    /**
     * 根据对象删除关系
     * 根据对象删除楼层的贯通关系，只针对一个对象
     * @param   param      对象
     * @return             删除的结果信息
     */

    @PostMapping("/unlink")
    public AdmResponse delete(@RequestParam String projectId,
                              @RequestBody JSONObject param) throws Exception {
        if(CollUtil.isEmpty(param)) {
            return AdmResponse.success();
        }
        // 组装上下文条件
        InstanceUrlParam context = new InstanceUrlParam(groupCode, null, projectId, AdmCommonConstant.APP_ID);
        // 组装条件
        QueryCriteria criteria = new QueryCriteria();
        String floorId = param.getString("floorId");
        String floorOtherId = param.getString("floorOtherId");
        if(!StrUtil.isAllNotEmpty(floorId,floorOtherId)) {
            return AdmResponse.failure("必填项：FloorId（楼层id）、FloorOtherId(其他楼层id)");
        }
        ObjectNode node = JsonNodeFactory.instance.objectNode();
        node.put("graphCode", "ThroughRelationship");
        node.put("relCode", "Fl2Fl");
        node.put("objFrom", floorId);
        node.put("objTo",floorOtherId);
        criteria.setCriteria(node);
        service.doDelete(context, criteria);
        return AdmResponse.success();
    }
    /**
     * 创建楼层的贯通关系
     * 楼层的贯通，楼层一对多
     * @param param 楼层和其他楼层列表关系对象r
     * @return
     * @throws Exception
     */
    @PostMapping("/link-or")
    public AdmResponse createFloorOtherIdList(@RequestParam("projectId") String projectId,
                                @RequestBody JSONObject param) throws Exception {
        String floorId = param.getString("floorId");
        JSONArray floorOtherIdList = param.getJSONArray("floorOtherIdList");
        if(StrUtil.isBlank(floorId) || CollUtil.isEmpty(floorOtherIdList)) {
            return AdmResponse.failure("必填项：FloorId（楼层id）、FloorOtherIdList（其他楼层id列表）,测方法会覆盖以前的记录");
        }
        // 创建关系对象.先删除，后添加
        InstanceUrlParam context = new InstanceUrlParam(groupCode, null, projectId, AdmCommonConstant.APP_ID);
        List<RelationDTO> voList = new ArrayList<>();
        for(int i = 0;i < floorOtherIdList.size();i++) {
            String floorOtherId = floorOtherIdList.getString(i);
            if(StrUtil.isBlank(floorOtherId)) {
                continue;
            }
            QueryCriteria queryRequest = new QueryCriteria();
            ObjectNode node = JsonNodeFactory.instance.objectNode();
            node.put("graphCode", "ThroughRelationship");
            node.put("relCode", "Fl2Fl");
            node.put("objFrom", floorId);
            node.put("objTo", floorOtherId);
            queryRequest.setCriteria(node);
            service.doDelete(context,queryRequest);
            // 创建关系对象 - 参考(datasyn中DataCenterSync.kt)
            voList.add(new RelationDTO(null, "ThroughRelationship", "Fl2Fl", null, floorId, floorOtherId));
        }
        // 组装上下文条件
        service.doSave(context, voList);
        return AdmResponse.success();
    }

    /**
     * 楼层的贯通，其他楼层一对多
     * @param param 楼层和其他楼层列表关系对象r
     * @return
     * @throws Exception
     */
    @PostMapping("/link-fl")
    public AdmResponse createFloorIdList(@RequestParam("projectId") String projectId,
                                              @RequestBody JSONObject param) throws Exception {
        String floorOtherId = param.getString("floorOtherId");
        JSONArray floorIdList = param.getJSONArray("floorIdList");
        if(StrUtil.isBlank(floorOtherId) || CollUtil.isEmpty(floorIdList)) {
            return AdmResponse.failure("必填项：FloorIdList（楼层id列表）、FloorOtherId（其他楼层id），测方法会覆盖以前的记录");
        }
        // 创建关系对象.先删除，后添加
        InstanceUrlParam context = new InstanceUrlParam(groupCode, null, projectId, AdmCommonConstant.APP_ID);
        List<RelationDTO> voList = new ArrayList<>();
        for(int i = 0;i < floorIdList.size();i++) {
            String floorId = floorIdList.getString(i);
            if(StrUtil.isBlank(floorId)) {
                continue;
            }
            QueryCriteria queryRequest = new QueryCriteria();
            ObjectNode node = JsonNodeFactory.instance.objectNode();
            node.put("graphCode", "ThroughRelationship");
            node.put("relCode", "Fl2Fl");
            node.put("objFrom", floorId);
            node.put("objTo", floorOtherId);
            queryRequest.setCriteria(node);
            service.doDelete(context,queryRequest);
            // 创建关系对象 - 参考(datasyn中DataCenterSync.kt)
            voList.add(new RelationDTO(null, "ThroughRelationship", "Fl2Fl", null, floorId, floorOtherId));
        }
        // 组装上下文条件
        service.doSave(context, voList);
        return AdmResponse.success();
    }

}
