package com.persagy.proxy.adm.service.impl;

import java.util.List;
import java.util.Map;
import java.util.Set;

import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;

import com.fasterxml.jackson.databind.ObjectMapper;
import com.fasterxml.jackson.databind.node.ObjectNode;
import com.google.common.collect.Lists;
import com.persagy.dmp.basic.dto.RequestData;
import com.persagy.dmp.basic.model.QueryCriteria;
import com.persagy.dmp.common.helper.SpringHelper;
import com.persagy.dmp.common.model.response.CommonResult;
import com.persagy.dmp.define.client.DigitalGraphDefineFacade;
import com.persagy.dmp.define.client.DigitalRelationDefineFacade;
import com.persagy.dmp.define.entity.GraphDefine;
import com.persagy.dmp.define.entity.RelationDefine;
import com.persagy.dmp.digital.client.DigitalObjectFacade;
import com.persagy.dmp.digital.client.DigitalRelationFacade;
import com.persagy.dmp.digital.client.DigitalRelationProjectCalFacade;
import com.persagy.dmp.digital.entity.ObjectRelation;
import com.persagy.dmp.simple.client.DigitalObjectSimpleFacade;
import com.persagy.proxy.adm.constant.AdmCommonConstant;
import com.persagy.proxy.adm.constant.AdmRelationTypeEnum;
import com.persagy.proxy.adm.constant.ObjTypeMapping;
import com.persagy.proxy.adm.service.IRelationReportService;
import com.persagy.proxy.adm.strategy.relationdata.RelationObjectStrategy;

import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.util.StrUtil;
import net.minidev.json.JSONObject;

/**
 * 报表/模板下载
 * 
 * @version 1.0.0
 * @company persagy 
 * @author zhangqiankun
 * @date 2021年8月31日 下午4:43:15
 */
@Service
public class RelationReportService implements IRelationReportService {

	@Value("${middleware.group.code}")
	private String defaultCode;

	@Override
	public List<Map<String, Object>> queryObjectCountByObjInfos(String groupCode, Set<String> classCodes, Set<String> objTypes, List<String> groups) {
		groupCode = StrUtil.isBlank(groupCode) ? defaultCode : groupCode;
		
		RequestData requestData = new RequestData();
		requestData.setClassCodes(classCodes);
		requestData.setObjTypes(objTypes);
		requestData.setGroups(groups);
		List<Map<String,Object>> list = DigitalObjectSimpleFacade.queryObjectCountByObjInfos(groupCode, AdmCommonConstant.APP_ID, AdmCommonConstant.USER_ID, requestData);
		return CollectionUtil.isEmpty(list) ? Lists.newArrayList() : list;
	}
	
	@Override
	public List<GraphDefine> getGraphDefine(String groupCode, String projectId, String graphCode) {
		groupCode = StrUtil.isBlank(groupCode) ? defaultCode : groupCode;
		
		ObjectMapper objectMapper = SpringHelper.getBean(ObjectMapper.class);
		QueryCriteria queryCriteria = new QueryCriteria();
		ObjectNode criteria = objectMapper.createObjectNode();
		if (StrUtil.isNotBlank(graphCode)) {
			criteria.put("code", graphCode);
		}
		queryCriteria.setCriteria(criteria);
		List<GraphDefine> graphDefines = DigitalGraphDefineFacade.query(groupCode, projectId, AdmCommonConstant.APP_ID, AdmCommonConstant.USER_ID, queryCriteria);
		return graphDefines;
	}
	
	@Override
	public List<RelationDefine> getRelationDefine(String groupCode, String projectId, String graphCode, String relCode) {
		groupCode = StrUtil.isBlank(groupCode) ? defaultCode : groupCode;
		
		ObjectMapper objectMapper = SpringHelper.getBean(ObjectMapper.class);
		QueryCriteria queryCriteria = new QueryCriteria();
		ObjectNode criteria = objectMapper.createObjectNode();
		if (StrUtil.isNotBlank(graphCode)) {
			criteria.put("graphCode", graphCode);
		}
		if (StrUtil.isNotBlank(relCode)) {
			criteria.put("code", relCode);
		}
		queryCriteria.setCriteria(criteria);
		List<RelationDefine> relationDefines = DigitalRelationDefineFacade.query(groupCode, projectId, AdmCommonConstant.APP_ID, AdmCommonConstant.USER_ID, queryCriteria);
		return relationDefines;
	}
	
	@Override
	public ObjectNode getRelationProjectCal(String groupCode, String projectId, String relType) {
		groupCode = StrUtil.isBlank(groupCode) ? defaultCode : groupCode;
		
		AdmRelationTypeEnum relationTypeEnum = AdmCommonConstant.RELATION_TYPE_MAP.get(relType);
		if (relationTypeEnum == null) {
			return null;
		}
		
		ObjectNode objectNode = this.getRelationProjectCal(groupCode, projectId, relationTypeEnum.getGraphCode(), relationTypeEnum.getRelCode(), null);
		if (objectNode != null) {
			String zoneType = objectNode.get("zoneType") == null ? null : objectNode.get("zoneType").asText();
			long count = this.countRelationObject(groupCode, projectId, relationTypeEnum.getGraphCode(), relationTypeEnum.getRelCode(), zoneType);
			objectNode.put("count", count);
			objectNode.put("relationType", relType);
			
			// 获取此图类型和边类型对应的数据记录
			List<RelationDefine> relationDefines = this.getRelationDefine(groupCode, projectId, relationTypeEnum.getGraphCode(), relationTypeEnum.getRelCode());
			if (CollectionUtil.isNotEmpty(relationDefines)) {
				RelationDefine relationDefine = relationDefines.get(0);
				objectNode.put("conneObject", relationDefine.getTargetObjs());
				objectNode.put("relationTypeCode", relationDefine.getCode());
				objectNode.put("relationTypeName", relationDefine.getName());
				objectNode.put("automatic", (null == relationDefine.getAutomaticFlag() || 0 == relationDefine.getAutomaticFlag()) ? false : true);
			}
			
			String prefix = ObjTypeMapping.getRelCodePrefix(relationTypeEnum.getRelCode());
			String suffix = ObjTypeMapping.getRelCodeSuffix(relationTypeEnum.getRelCode());
			objectNode.put("mainObject", ObjTypeMapping.PREFIX_NAME.get(prefix));
			objectNode.put("fromObject", ObjTypeMapping.PREFIX_NAME.get(suffix));
			objectNode.putPOJO("statistics", new JSONObject());
		}
		
		return objectNode;
	}
	
	@Override
	public ObjectNode getRelationProjectCal(String groupCode, String projectId, String graphCode, String relCode, String zoneType) {
		groupCode = StrUtil.isBlank(groupCode) ? defaultCode : groupCode;
		
		ObjectMapper objectMapper = SpringHelper.getBean(ObjectMapper.class);
		QueryCriteria queryCriteria = new QueryCriteria();
		ObjectNode criteria = objectMapper.createObjectNode();
		criteria.put("graphCode", graphCode);
		criteria.put("relCode", relCode);
		if (StrUtil.isNotBlank(zoneType)) {
			criteria.put("zoneType", zoneType);
		}
		queryCriteria.setCriteria(criteria);
		List<ObjectNode> objectNodes = DigitalRelationProjectCalFacade.query(groupCode, projectId, AdmCommonConstant.APP_ID, AdmCommonConstant.USER_ID, queryCriteria);
		if (CollectionUtil.isEmpty(objectNodes)) {
			return null;
		}
		return objectNodes.get(0);
	}
	
	@Override
	public long countRelationObject(String groupCode, String projectId, String graphCode, String relCode, String zoneType) {
		groupCode = StrUtil.isBlank(groupCode) ? defaultCode : groupCode;
		
		QueryCriteria queryCriteria = new QueryCriteria();
		ObjectMapper objectMapper = SpringHelper.getBean(ObjectMapper.class);
		ObjectNode criteria = objectMapper.createObjectNode();
		criteria.put("graphCode", graphCode);
		criteria.put("relCode", relCode);
		if (StrUtil.isNotBlank(zoneType)) {
			criteria.put("relValue", zoneType);
		}
		queryCriteria.setCriteria(criteria);
		queryCriteria.setOnlyCount(true);
		CommonResult<List<ObjectRelation>> result = DigitalRelationFacade.queryPrototype(groupCode, projectId, AdmCommonConstant.APP_ID, AdmCommonConstant.USER_ID, queryCriteria);
		return result == null ? 0 : (result.getCount() == null ? 0 : result.getCount());
	}
	
	@Override
	public long countObject(String groupCode, String projectId, String classCode, String objType) {
		groupCode = StrUtil.isBlank(groupCode) ? defaultCode : groupCode;
		
		QueryCriteria queryCriteria = new QueryCriteria();
		ObjectMapper objectMapper = SpringHelper.getBean(ObjectMapper.class);
		ObjectNode criteria = objectMapper.createObjectNode();
		criteria.put("classCode", classCode);
		if (StrUtil.isNotBlank(objType)) {
			criteria.put("objType", objType);
		}
		queryCriteria.setCriteria(criteria);
		queryCriteria.setOnlyCount(true);
		CommonResult<List<ObjectNode>> result = DigitalObjectFacade.queryPrototype(groupCode, projectId, AdmCommonConstant.APP_ID, AdmCommonConstant.USER_ID, queryCriteria);
		return result == null ? 0 : (result.getCount() == null ? 0 : result.getCount());
	}
	
	@Override
	public ObjectNode getObjectNode(String groupCode, String projectId, String id) {
		groupCode = StrUtil.isBlank(groupCode) ? defaultCode : groupCode;
		
		ObjectMapper objectMapper = SpringHelper.getBean(ObjectMapper.class);
		QueryCriteria queryCriteria = new QueryCriteria();
		ObjectNode criteria = objectMapper.createObjectNode();
		criteria.put("id", id);
		queryCriteria.setCriteria(criteria);
		List<ObjectNode> objectNodes = DigitalObjectFacade.query(groupCode, projectId, AdmCommonConstant.APP_ID, AdmCommonConstant.USER_ID, queryCriteria);
		return CollectionUtil.isEmpty(objectNodes) ? objectMapper.createObjectNode() : objectNodes.get(0); 
	}
	
	@Override
	public List<ObjectNode> findObjectNodes(String groupCode, String projectId, String objType, String code, String value) {
		groupCode = StrUtil.isBlank(groupCode) ? defaultCode : groupCode;
		
		if (StrUtil.isBlank(code)) {
			return null;
		}
		String key = RelationObjectStrategy.OBJNAME_2_TYPE.get(code);
		if (StrUtil.isBlank(key)) {
			return null;
		}
		
		ObjectMapper objectMapper = SpringHelper.getBean(ObjectMapper.class);
		QueryCriteria queryCriteria = new QueryCriteria();
		ObjectNode criteria = objectMapper.createObjectNode();
		criteria.put(key, value);
		criteria.put("objType", objType);
		queryCriteria.setCriteria(criteria);
		List<ObjectNode> objectNodes = DigitalObjectFacade.query(groupCode, projectId, AdmCommonConstant.APP_ID, AdmCommonConstant.USER_ID, queryCriteria);
		return objectNodes;
	}
	
	@Override
	public ObjectNode getObjectByCodeAndId(String groupCode, String projectId, String graphCode, String relCode, String objFrom, String objTo) {
		groupCode = StrUtil.isBlank(groupCode) ? defaultCode : groupCode;
		
		// 获取出关联对象
		ObjectMapper objectMapper = SpringHelper.getBean(ObjectMapper.class);
		
		QueryCriteria queryCriteria = new QueryCriteria();
		ObjectNode criteria = objectMapper.createObjectNode();
		criteria.put("graphCode", graphCode);
		criteria.put("relCode", relCode);
		criteria.put("objFrom", objFrom);
		criteria.put("objTo", objTo);
		queryCriteria.setCriteria(criteria);
		List<ObjectRelation> objectRelations = DigitalRelationFacade.query(groupCode, projectId, AdmCommonConstant.APP_ID, AdmCommonConstant.USER_ID, queryCriteria);
		if (CollectionUtil.isEmpty(objectRelations)) {
			return objectMapper.createObjectNode();
		}
		
		// 获取关联对象的本地名称
		ObjectRelation objectRelation = objectRelations.get(0);
		String relationId = StrUtil.isBlank(objFrom) ? objectRelation.getObjFrom() : objectRelation.getObjTo();
		return this.getObjectNode(groupCode, projectId, relationId); 
	}
	
	@Override
	public List<ObjectRelation> findObjectRelationList(String groupCode, String projectId, String graphCode, String relCode) {
		groupCode = StrUtil.isBlank(groupCode) ? defaultCode : groupCode;
		
		// 获取出关联对象
		ObjectMapper objectMapper = SpringHelper.getBean(ObjectMapper.class);
		
		QueryCriteria queryCriteria = new QueryCriteria();
		ObjectNode criteria = objectMapper.createObjectNode();
		criteria.put("graphCode", graphCode);
		criteria.put("relCode", relCode);
		queryCriteria.setCriteria(criteria);
		List<ObjectRelation> objectRelations = DigitalRelationFacade.query(groupCode, projectId, AdmCommonConstant.APP_ID, AdmCommonConstant.USER_ID, queryCriteria);
		return CollectionUtil.isEmpty(objectRelations) ? Lists.newArrayList() : objectRelations;
	}
	
}
